@@ -6,6 +6,7 @@ import requests
 import pandas as pd
 from IPython.display import display, HTML
 
+
 logger = logging.getLogger(__name__)
 
 STATUS = "status"
@@ -29,6 +30,7 @@ API_GET_ATHENA_DATABASES = "/ccf/services/v2.0/get_athena_databases"
 API_GET_ATHENA_TABLES = "/ccf/services/v2.0/get_athena_tables"
 API_QUERY_ATHENA_TABLE = "/ccf/services/v2.0/query_athena_table"
 API_RUN_QUERY_ON_ATHENA = "/ccf/services/v2.0/run_query_on_athena"
+API_SAVE_DATA_TO_S3 = "/ccf/services/v2.0/save_data_to_s3"
 
 # Environment Variables
 _USER_NAME: str = 'USER_NAME'
@@ -95,12 +97,13 @@ class AthenaClient:
                               cookies=dict(sessionId=self._access_token))
             response = r.json()
             if response[STATUS] == SUCCESS:
-                if result_as_df and isinstance(response[RESULT], (dict, list)):
+                if not result_as_df and isinstance(response[RESULT], (dict, list)):
                     response_as_df = pd.DataFrame.from_dict(response[RESULT]).style.set_caption(
                         "Databases").hide_index().set_table_styles(TABLE_STYLES)
                     return display(response_as_df)
                 else:
-                    return response[RESULT]
+                    response_as_df = pd.DataFrame.from_dict(response[RESULT])
+                    return response_as_df
             else:
                 return response[ERROR]
 
@@ -121,12 +124,13 @@ class AthenaClient:
                               cookies=dict(sessionId=self._access_token))
             response = r.json()
             if response[STATUS] == SUCCESS:
-                if result_as_df and isinstance(response[RESULT], (dict, list)):
+                if not result_as_df and isinstance(response[RESULT], (dict, list)):
                     response_as_df = pd.DataFrame.from_dict(response[RESULT]).style.set_caption(
                         "Tables").hide_index().set_table_styles(TABLE_STYLES)
                     return display(response_as_df)
                 else:
-                    return response[RESULT]
+                    response_as_df = pd.DataFrame.from_dict(response[RESULT])
+                    return response_as_df
             else:
                 return response[ERROR]
 
@@ -148,13 +152,14 @@ class AthenaClient:
                               cookies=dict(sessionId=self._access_token))
             response = r.json()
             if response[STATUS] == SUCCESS:
-                if result_as_df and isinstance(response[RESULT], (dict, list)):
+                if not result_as_df and isinstance(response[RESULT], (dict, list)):
                     table_header = f'Top {limit} records of table {schema_name}.{table_name}' if limit else f'Top 100 records of table <i>{schema_name}.{table_name}</i>'
                     response_as_df = pd.DataFrame.from_dict(response[RESULT]).style.set_caption(
                         table_header).hide_index().set_table_styles(TABLE_STYLES)
                     return display(response_as_df)
                 else:
-                    return response[RESULT]
+                    response_as_df = pd.DataFrame.from_dict(response[RESULT])
+                    return response_as_df
             else:
                 return response[ERROR]
 
@@ -189,7 +194,32 @@ class AthenaClient:
                             TABLE_STYLES)
                         return display(result)
                 else:
-                    return response[RESULT]
+                    results = [pd.DataFrame.from_dict(item) for item in response[RESULT]]
+                    return results
+            else:
+                return response[ERROR]
+
+        except requests.exceptions.HTTPError:
+            raise Exception("Http error occurred") from None
+        except requests.exceptions.ConnectionError:
+            raise ConnectionError("Connection error occurred") from None
+        except requests.exceptions.Timeout:
+            raise TimeoutError("Request timed out...") from None
+        except requests.exceptions.RequestException:
+            raise Exception("RequestException occurred") from None
+
+    def save_data(self, data, file_name, s3_path):
+        """Returns records from a table"""
+        try:
+            d = data.to_dict(orient='records')
+            print(d)
+            payload = json.dumps({"user_name": self._user_name, "role_id": self._user_role, "file_name": file_name,
+                                  "s3_bucket": s3_path, "data": d})
+            r = requests.post(self._ccf_services_host + API_SAVE_DATA_TO_S3, data=payload, headers=HEADERS,
+                              cookies=dict(sessionId=self._access_token))
+            response = r.json()
+            if response[STATUS] == SUCCESS:
+                return response[RESULT]
             else:
                 return response[ERROR]
 
